package org.apache.bcel.generic;

import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.ExceptionConst;
import org.apache.bcel.ExceptionConst.EXCS;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.util.ByteSequence;

public class MULTIANEWARRAY extends CPInstruction implements LoadClass, AllocationInstruction, ExceptionThrower {
   private short dimensions;

   MULTIANEWARRAY() {
   }

   public MULTIANEWARRAY(int index, short dimensions) {
      super((short)197, index);
      if (dimensions < 1) {
         throw new ClassGenException("Invalid dimensions value: " + dimensions);
      } else {
         this.dimensions = dimensions;
         super.setLength(4);
      }
   }

   public void accept(Visitor v) {
      v.visitLoadClass(this);
      v.visitAllocationInstruction(this);
      v.visitExceptionThrower(this);
      v.visitTypedInstruction(this);
      v.visitCPInstruction(this);
      v.visitMULTIANEWARRAY(this);
   }

   public int consumeStack(ConstantPoolGen cpg) {
      return this.dimensions;
   }

   public void dump(DataOutputStream out) throws IOException {
      out.writeByte(super.getOpcode());
      out.writeShort(super.getIndex());
      out.writeByte(this.dimensions);
   }

   public final short getDimensions() {
      return this.dimensions;
   }

   public Class<?>[] getExceptions() {
      return ExceptionConst.createExceptions(EXCS.EXCS_CLASS_AND_INTERFACE_RESOLUTION, new Class[]{ExceptionConst.ILLEGAL_ACCESS_ERROR, ExceptionConst.NEGATIVE_ARRAY_SIZE_EXCEPTION});
   }

   public ObjectType getLoadClassType(ConstantPoolGen cpg) {
      Type t = this.getType(cpg);
      if (t instanceof ArrayType) {
         t = ((ArrayType)t).getBasicType();
      }

      return t instanceof ObjectType ? (ObjectType)t : null;
   }

   protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
      super.initFromFile(bytes, wide);
      this.dimensions = (short)bytes.readByte();
      super.setLength(4);
   }

   public String toString(boolean verbose) {
      return super.toString(verbose) + " " + super.getIndex() + " " + this.dimensions;
   }

   public String toString(ConstantPool cp) {
      return super.toString(cp) + " " + this.dimensions;
   }
}
